@@ -2,7 +2,7 @@
  * debug.c - Print debug information to stdout.
  *           This file is part of the FreeLCD package.
  *
- * $Id: debug.c,v 1.2 2004/01/25 15:46:56 unicorn Exp $
+ * $Id: debug.c,v 1.3 2004/06/20 12:32:31 unicorn Exp $
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -54,7 +54,19 @@ void message (const char *format, ...)
 
 void set_debug_output (FILE *handle)
 {
+  close_debug_output ();
   _handle = handle;
 }
 
-#endif /* USE_DEBUG */
+void close_debug_output (FILE *handle)
+{
+  if (_handle != NULL)
+    fclose (_handle);
+}
+
+#else /* !USE_DEBUG */
+
+void message (const char *format, ...)
+{}
+
+#endif /* !USE_DEBUG */
