@@ -2,7 +2,7 @@
  *  xmlt.c - XML data structure.
  *           This file is part of the FreeLCD package.
  *
- *  $Id: xmlt.c,v 1.8 2004/01/25 15:48:20 unicorn Exp $
+ *  $Id: xmlt.c,v 1.9 2004/06/20 12:38:35 unicorn Exp $
  *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -41,8 +41,6 @@
 #include <stdio.h>
 
 #include "dictionary.h"
-#include "slist.h"
-#include "xmalloc.h"
 #include "entities.h"
 #include "xmlt.h"
 #include "debug.h"
@@ -73,10 +71,10 @@ xml_context;
 static xml_node *
 _create_xml_node (xml_node_t type, int tag)
 {
-  xml_node *new_node = xmalloc (sizeof (xml_node));
+  xml_node *new_node = g_malloc (sizeof (xml_node));
 
-  slist_init (&new_node->children);
-  slist_init (&new_node->attributes);
+  new_node->children = NULL;
+  new_node->attributes = NULL;
 
   new_node->type = type;
   new_node->tag = tag;
@@ -88,17 +86,14 @@ _create_xml_node (xml_node_t type, int tag)
 
 /*----------------------------------------------------- _cleanup_string --*/
 static size_t
-_cleanup_string (const char *text, char **_dest, size_t len)
+_cleanup_string (const char *text, char *dest, size_t len)
 {
   size_t i;
-  size_t j;
   size_t newlen = 0;
-  unsigned int unicode;
   char c;
-  char* dest = *_dest;
 
   assert (text != NULL);
-  assert (_dest != NULL && *_dest != NULL);
+  assert (dest != NULL);
 
   if (len == 0)
     return 0;
@@ -122,18 +117,18 @@ _cleanup_string (const char *text, char **_dest, size_t len)
 }
 
 /*------------------------------------------------------- _attr_compare --*/
-static int
-_attr_compare (const void* data, const void* compare)
+static gint
+_attr_compare (gconstpointer data, gconstpointer compare)
 {
   const xml_attribute *attr = (const xml_attribute*)data;
   int attr_t = *(int*)compare;
 
-  return attr->attribute == attr_t;
+  return attr->attribute == attr_t ? 0 : 1;
 }
 
 /*----------------------------------------------------- _rescan_attribs --*/
 static void
-_rescan_attrib (void *data, void *userdata)
+_rescan_attrib (gpointer data, gpointer userdata)
 {
   xml_attribute *attrib = (xml_attribute*)data; 
   dictionary *attr_dict = (dictionary*)userdata;
@@ -145,12 +140,13 @@ _rescan_attrib (void *data, void *userdata)
       if (lookup != NULL)
         {
           attrib->attribute = *lookup;
-          free (attrib->unknown_attrib);
+          g_free (attrib->unknown_attrib);
           attrib->unknown_attrib = NULL;
         }
     }
 }
 
+/*------------------------------------------------------ _handle_entity --*/
 static int
 _handle_entity (XML_Parser p, const XML_Char *context, const XML_Char *base,
                 const XML_Char *sys_id, const XML_Char *publicid)
@@ -165,7 +161,6 @@ _start_element (void *cntxt, const char *name, const char **attr)
   xml_node *new_node = _create_xml_node (NODE, XML_UNKNOWN_TAG);
   xml_context *context = (xml_context *)cntxt;
   int *node_t;
-
   
   /*  Look up the node name in the dictionary. If it's not there, mark
    *  it simply as an unknown node.
@@ -173,7 +168,7 @@ _start_element (void *cntxt, const char *name, const char **attr)
   node_t = dict_lookup (context->known_tags, name);
   if (node_t == NULL)
     {
-      new_node->cdata = xstrdup (name);
+      new_node->cdata = g_strdup (name);
     }
   else
     {
@@ -189,12 +184,13 @@ _start_element (void *cntxt, const char *name, const char **attr)
   else
     {
       new_node->parent = context->curr_node;
-      slist_append (&context->curr_node->children, new_node);
+      context->curr_node->children = 
+        g_slist_append (context->curr_node->children, new_node);
     }
 
   while (attr != NULL && *attr != NULL)
     {
-      xml_attribute *new_attr = xmalloc (sizeof (xml_attribute));
+      xml_attribute *new_attr = g_malloc (sizeof (xml_attribute));
   
       if (node_t == NULL)
         {
@@ -203,7 +199,7 @@ _start_element (void *cntxt, const char *name, const char **attr)
            * other modules that rescan subsections of the document. */
 
           new_attr->attribute = XML_UNKNOWN_ATTRIBUTE;
-          new_attr->unknown_attrib = xstrdup (*attr);
+          new_attr->unknown_attrib = g_strdup (*attr);
         }
       else
         {
@@ -212,7 +208,7 @@ _start_element (void *cntxt, const char *name, const char **attr)
           if (attr_t == NULL)
             {
               new_attr->attribute = XML_UNKNOWN_ATTRIBUTE;
-              new_attr->unknown_attrib = xstrdup (*attr);
+              new_attr->unknown_attrib = g_strdup (*attr);
             }
           else
             {
@@ -221,8 +217,8 @@ _start_element (void *cntxt, const char *name, const char **attr)
             }
         }
       
-      new_attr->value = xstrdup(*(attr + 1));
-      slist_append (&new_node->attributes, new_attr);
+      new_attr->value = g_strdup(*(attr + 1));
+      new_node->attributes = g_slist_append (new_node->attributes, new_attr);
       attr += 2;
     }
   
@@ -242,7 +238,9 @@ _end_element (void *cntxt, const char *name)
   --context->depth;
   if (context->depth == 0)
     {
+      debug ("xml doc completed");
       context->doc_finished (context->user_callback_data, context->root_node);
+      context->root_node = NULL;
       context->reset = 1;
     }
   else
@@ -260,21 +258,20 @@ _character_data (void *cntxt, const char *text, int len)
   char        *newtext;
   xml_node    *new_node;
   xml_context *context = (xml_context *) cntxt;
-  slist       *siblings;
+  GSList     **siblings = NULL;
 
   assert (cntxt != NULL);
   assert (text != NULL);
   
-  newtext = xmalloc (len);
-  newlen = _cleanup_string (text, &newtext, len);
+  newtext = g_malloc (len + 1); /* plus one for 0 character */
+  newlen = _cleanup_string (text, newtext, len);
+  assert (newlen <= len);
   if (newlen == 0)
     {
-      free (newtext);
+      g_free (newtext);
       return;
     }
   
-
-  newtext = xrealloc (newtext, newlen + 1);
   newtext[newlen] = '\0';
 
   siblings = &(context->curr_node->children);
@@ -283,11 +280,12 @@ _character_data (void *cntxt, const char *text, int len)
       /*  The parser was already handling character data. Just add this
        *  cdata to what was already there.
        */
-      xml_node *last_node = slist_last (siblings);
+      xml_node *last_node = (xml_node*) g_slist_last(*siblings)->data;
       size_t cdata_len = strlen (last_node->cdata);
 
-      last_node->cdata = xrealloc (last_node->cdata, cdata_len + newlen + 1);
+      last_node->cdata = g_realloc (last_node->cdata, cdata_len + newlen + 1);
       memcpy (last_node->cdata + cdata_len, newtext, newlen);
+      g_free (newtext);
       last_node->cdata[cdata_len + cdata_len + newlen + 1] = '\0';
     }
   else
@@ -296,20 +294,21 @@ _character_data (void *cntxt, const char *text, int len)
       new_node->cdata = newtext;
       new_node->parent = context->curr_node;
   
-      slist_append (siblings, new_node);
+      *siblings = g_slist_append (*siblings, new_node);
       context->curr_is_cdata = 1;
     }
 }
 
 /*----------------------------------------------------- _free_attribute --*/
 static void
-_free_attribute (void *_attr)
+_free_attribute (gpointer _attr, gpointer userdata)
 {
   xml_attribute *attr = (xml_attribute *) _attr;
+  (void)userdata;
   assert (attr != NULL);
-  free (attr->value);
-  free (attr->unknown_attrib);
-  free (attr);
+  g_free (attr->value);
+  g_free (attr->unknown_attrib);
+  g_free (attr);
 }
 
 static void
@@ -335,7 +334,7 @@ void *
 xmlt_create_context (void (*cb) (void *, xml_node *), void *cb_data,
                      dictionary *tags, dictionary *attribs)
 {
-  xml_context *context = xmalloc (sizeof (xml_context));
+  xml_context *context = g_malloc (sizeof (xml_context));
 
   debug ("xmlt_create_context");
   context->parser = XML_ParserCreate (0);
@@ -356,7 +355,6 @@ xmlt_create_context (void (*cb) (void *, xml_node *), void *cb_data,
 int
 xmlt_parse (void *_context, const char *data, size_t len)
 {
-  const char *start = data;
   xml_context *context = (xml_context *) _context;
   
   assert (context != NULL);
@@ -379,8 +377,8 @@ xmlt_parse (void *_context, const char *data, size_t len)
                * of thing FreeLCD uses.  Try to reset the parser context and
                * continue where the error manifested. */
 
-              size_t index = XML_GetCurrentByteIndex (context->parser);
-              size_t skip = index - context->parsed;
+              size_t idx = XML_GetCurrentByteIndex (context->parser);
+              size_t skip = idx - context->parsed;
               len -= skip;
               data += skip;
 
@@ -399,8 +397,8 @@ xmlt_parse (void *_context, const char *data, size_t len)
         }
       else if (context->reset)
         {
-          size_t index = XML_GetCurrentByteIndex (context->parser);
-          size_t skip = index - context->parsed;
+          size_t idx = XML_GetCurrentByteIndex (context->parser);
+          size_t skip = idx - context->parsed;
           len -= skip;
           data += skip;
 
@@ -448,33 +446,43 @@ xmlt_reset_context (void *_context)
 
 /*--------------------------------------------------- xmlt_free_context --*/
 void
-xmlt_free_context (void *context)
+xmlt_free_context (void *_context)
 {
+  xml_context *context = (xml_context *)_context;
+  
+  if (context == NULL)
+    return;
+
   debug ("xmlt_free_context");
-  assert (context);
-  XML_ParserFree (((xml_context *) context)->parser);
+  xmlt_free_document (context->root_node);
+  XML_ParserFree (context->parser);
+  g_free (context);
 }
 
 /*-------------------------------------------------- xmlt_free_document --*/
 void
 xmlt_free_document (xml_node *doc)
 {
-  slist_iter iter;
+  GSList *iter;
   int is_cdata = 1;
   void *p;
 
-  assert (doc != NULL);
+  if (doc == NULL)
+    return;
 
   debug ("xmlt_free_document");
-  iter = slist_begin_iter (&doc->children);
-  slist_delete_special (&doc->attributes, _free_attribute);
+  iter = doc->children;
+  g_slist_foreach (doc->attributes, _free_attribute, NULL);
+  g_slist_free (doc->attributes);
+  doc->attributes = NULL;
 
-  while (iter.curr)
+  while (iter != NULL)
     {
-      p = slist_iter_and_next (&iter);
+      p = iter->data;
+      iter = g_slist_next (iter);
 
       if (is_cdata)
-        free (p);
+        g_free (p);
       else
         xmlt_free_document (p);
 
@@ -482,30 +490,30 @@ xmlt_free_document (xml_node *doc)
       is_cdata = -is_cdata + 1;
     }
 
-  free (doc->cdata);
-  free (doc);
+  g_free (doc->cdata);
+  g_free (doc);
 }
 
 /*------------------------------------------------------- xmlt_get_next --*/
 xml_node *
 xmlt_get_next (xml_node *iter, xml_node *doc)
 {
-  slist_iter i;
+  GSList *i;
 
   assert (iter != NULL);
   assert (doc != NULL);
   
-  if (iter->children.head != NULL)
-    return (xml_node*)iter->children.head->data;
+  if (iter->children != NULL)
+    return (xml_node*)iter->children->data;
   
   while (iter->parent != NULL)
     {
       if (iter == doc)
         return NULL;
 
-      i = slist_find (&(iter->parent->children), iter);
-      if (i.curr->next)
-        return (xml_node *) (i.curr->next->data);
+      i = g_slist_find (iter->parent->children, iter);
+      if (i->next != NULL)
+        return (xml_node *) (i->next->data);
 
       iter = iter->parent;
     }
@@ -518,13 +526,13 @@ xmlt_get_next (xml_node *iter, xml_node *doc)
 xml_node *
 xmlt_get_next_shallow (xml_node *iter)
 {
-  slist_iter siblings;
+  GSList *siblings;
   assert (iter);
   
-  siblings = slist_find (&(iter->parent->children), iter);
+  siblings = g_slist_find (iter->parent->children, iter);
 
-  if (siblings.curr->next)
-    iter = (xml_node *)(siblings.curr->next->data);
+  if (siblings->next != NULL)
+    iter = (xml_node *)(siblings->next->data);
   else
     iter = NULL;
 
@@ -553,13 +561,13 @@ xmlt_rescan_document (xml_node *doc, dictionary *tags, dictionary *attribs)
           if (tag_ptr != NULL)
             {
               curr_node->tag = *tag_ptr;
-              free (curr_node->cdata);
+              g_free (curr_node->cdata);
               curr_node->cdata = NULL;
 
               if (attribs != NULL)
                 {
-                  slist_for_each (&curr_node->attributes, _rescan_attrib, 
-                                  attribs);
+                  g_slist_foreach (curr_node->attributes, _rescan_attrib, 
+                                   attribs);
                 }
             }
         }
@@ -619,10 +627,11 @@ xmlt_for_each (xml_node *doc, void *user_data, void (*func)(void*, xml_node*))
 const char*
 xmlt_get_attrib (xml_node *node, int attribute)
 {
-  slist_iter i = slist_find_if (&node->attributes, &attribute, _attr_compare);
+  GSList *i;
   
-  if (i.curr)
-    return ((xml_attribute*)i.curr->data)->value;
+  i = g_slist_find_custom (node->attributes, &attribute, _attr_compare);
+  if (i != NULL)
+    return ((xml_attribute*)i->data)->value;
 
   return NULL;
 } 
@@ -633,10 +642,10 @@ xmlt_get_first_cdata (xml_node *node)
 {
   xml_node *first_child;
 
-  if (node == NULL || node->children.head == NULL)
+  if (node == NULL || node->children == NULL)
     return NULL;
   
-  first_child = (xml_node*)node->children.head->data;
+  first_child = (xml_node*)node->children->data;
 
   return first_child ? first_child->cdata : NULL;
 }
@@ -644,8 +653,8 @@ xmlt_get_first_cdata (xml_node *node)
 
 #ifdef UNIT_TEST_XMLT_C
 
-/* UNIT_CFLAGS -O2 -lexpat */
-/* UNIT_EXTRA  ../common/xmalloc.c ../common/slist.c ../common/dictionary.c */
+/* UNIT_CFLAGS -O2 -lexpat -lglib-2.0 -I/usr/include/glib-2.0 -I/usr/lib/glib-2.0/include */
+/* UNIT_EXTRA  ../common/dictionary.c */
 
 void test_callback (void *userdata, xml_node *document)
 {
