@@ -2,7 +2,7 @@
  *  layout_tags.h : Some XML tags that are shared between several files.
  *                  This file is part of the FreeLCD package.
  *
- *  $Id: layout_tags.h,v 1.2 2004/01/16 23:39:41 unicorn Exp $
+ *  $Id: layout_tags.h,v 1.3 2004/06/20 14:44:51 unicorn Exp $
  *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -32,15 +32,15 @@
 
 typedef enum
 {
-  ROW = 1, COL, FIELD, LABEL
+  ROW = 200, COL, FIELD, LABEL, HSCROLL, VSCROLL
 } 
 tag_t;
 
 typedef enum
 {
-  TYPE = 1, ID, NAME, VALIGN, HALIGN, EXPAND
+  TYPE = 200, ID, NAME, VALIGN, HALIGN, EXPAND, FORMAT, TIMEZONE,
+  SCROLL
 }
 attr_t;
 
 #endif
-
