@@ -2,7 +2,7 @@
  *  connections.h - Manage connected sockets and the incoming data.
  *                  This file is part of the FreeLCD package.
  *
- *  $Id: connections.h,v 1.4 2003/11/05 21:33:04 unicorn Exp $
+ *  $Id: connections.h,v 1.5 2004/06/20 13:02:09 unicorn Exp $
  *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -25,11 +25,13 @@
 #ifndef _FLCDD_CONNECTIONS_H
 #define _FLCDD_CONNECTIONS_H 1
 
+#include <glib.h>
 #include <netinet/in.h>
 
-void conn_open (int sock, struct sockaddr_in *s);
-void conn_data (int sock, char *buf, size_t size);
-void conn_close (int sock);
+gboolean conn_open (GIOChannel *source, GIOCondition condition, gpointer data);
+gboolean conn_data (GIOChannel *source, GIOCondition condition, gpointer data); 
+gboolean conn_close (GIOChannel *source, GIOCondition condition, gpointer data);
+void conn_close_all ();
 int conn_reply (void *originator, const char *data, size_t len);
 
 #endif /* Multiple include guard */
