@@ -2,7 +2,7 @@
  *  drivers.h - Loading and managing the driver plugins.
  *              This file is part of the FreeLCD package.
  *
- *  $Id: drivers.h,v 1.5 2003/03/23 20:14:11 unicorn Exp $
+ *  $Id: drivers.h,v 1.6 2004/06/20 14:14:09 unicorn Exp $
  *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -29,7 +29,13 @@
 #include "common/xmlt.h"
 #include "drivers/driver.h"
 
-int drivers_init ();
+/* Initialize the driver management module. */
+int 
+drivers_init ();
+
+/* Clean up all resources taken by the driver management module. */
+void 
+drivers_cleanup ();
 
 /* Open a driver */
 void* 
