@@ -2,7 +2,7 @@
  *  guile.h - Routines for setting up libguile.
  *            This file is part of the FreeLCD package.
  *
- *  $Id: guile.h,v 1.1 2003/11/05 22:04:14 unicorn Exp $
+ *  $Id: guile.h,v 1.2 2004/06/20 14:16:19 unicorn Exp $
  *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -26,6 +26,8 @@
 #ifndef _FLCDD_GUILE_H
 #define _FLCDD_GUILE_H 1
 
+#include <glib.h>
+
 typedef enum
 {
   HOOK_NEXT_SCREEN      = 0, 
@@ -35,10 +37,20 @@ typedef enum
 }
 guile_hook;
 
-int
+void
 guile_init ();
 
+void
+guile_cleanup ();
+
 void
 guile_call_hook (guile_hook id, void *data, long param1, long param2);
 
+GSList *
+guile_get_time_layout_hints (const char *format);
+
+char*
+guile_format_time (const char *format, int width, int height,
+                   int utctime, int loctime);
+
 #endif /* Multiple include guard */
